/***************************************************************************
                    patterns.h  -  stone group patterns
                             -------------------
    begin                : Thu Sep 19 2002
    copyright            : (C) 2002 by Andrew D. Balsa
    email                : andrebalsa@mailingaddress.org
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

/****************  Defines  *******************/

/*flags in shape table entries for types of points*/

#define F_BLACK 1       /* the point has a black stone */
#define F_WHITE 2       /* the point has a white stone */
#define F_EMPTY 4       /* the point is empty */
#define F_OFF   8       /* the point is off the board */

#define PATTERN 12345	/* This integer should appear at the start of each */
			/* pattern.  If it doesn't, */
			/* someone has made a typographical error.*/

#define PATTEND 7171	/* This integer should appear at the end of the */
			/* table. */

/****************  Patterns  *******************/

int patterns[]=
{  PATTERN,     /*a code for the beginning of a pattern*/
   4, 24,       /*Three points, basic urgency 24.  Urgency is highest */
                /* for lower numbers.  Original values: */
                /*   capturing an enemy group=16, */
                /*   defending one of ours= 20, */
                /*   atariing an enemy group= 32 */
  -1, 0, F_BLACK|F_OFF, /*Recognize the pattern by black stone at (x-1,
y+0) */
                /* or that point off the board, */
   1, 0, F_BLACK,  /* a black stone at (x+1, y+0), and */
   0, -1, F_WHITE, /* a white stone at (x+0, y-1), and */
   0, 1, F_EMPTY|F_WHITE, /* a white stone or space at (x, y+1), */
        /* that is: */  /*   ~   */
                        /* ~ $ # */
                        /*   O   */

  PATTERN, 3, 22,       /* # $   */
  -1, 0, F_BLACK,       /*   O # */
   1, -1, F_BLACK,
   0, -1, F_WHITE,

  PATTERN, 5, 26,       /* # O # */
  -1, 1, F_BLACK|F_OFF, /*   $ . */
   1, 1, F_BLACK,       /*       */
   0, 1, F_WHITE,       /*   ~   */
   1, 0, F_EMPTY,
   0, -3, F_EMPTY|F_WHITE,      /*This so that we don't get trapped in */
                        /* an extremely common pattern against the edge */
                        /* of the board, or in a shicho.*/

  PATTERN, 4, 26,       /* # O # */
  -1, 1, F_BLACK|F_OFF, /*   $ # */
  1, 1, F_BLACK,
  1, 0, F_BLACK,
  0, 1, F_WHITE,

  PATTERN, 6, 24,       /*   ~     */
  -1, 0, F_BLACK,       /* # $ . # */
  0, -1, F_WHITE,       /*   O     */
  1, 0,  F_EMPTY,
  1, -1, F_EMPTY,
  2, 0,  F_BLACK,
  0, 1, F_EMPTY|F_WHITE,

  PATTERN, 5, 27,       /*   ~     */
  -1, 0, F_BLACK,       /* # $ . ~ */
  0, -1, F_WHITE,       /*   O     */
  1, 0,  F_EMPTY,
  2, 0,  F_OFF,
  0, 1, F_EMPTY|F_WHITE,

  PATTERN, 5, 24,       /* # . $ . # */
  -2, 0, F_BLACK,       /*     O     */
  -1, 0, F_EMPTY,
  0, -1, F_WHITE,
  1,  0, F_EMPTY,
  2,  0, F_BLACK,

  PATTERN, 5, 30,       /*  # . $ . ~  */
  -2, 0, F_BLACK,       /*      O      */
  -1, 0, F_EMPTY,
  0, -1, F_WHITE,
  1,  0, F_EMPTY,
  2,  0, F_OFF,

  PATTERN, 7, 26,               /*    ~ .    */
   -1, 0, F_BLACK,              /*  # $ # O  */
   1, -1, F_WHITE,              /*    ~ O    */
   1, 0, F_BLACK,
   2, 0, F_WHITE,
   1, 1, F_EMPTY,
   0, 1, F_EMPTY|F_WHITE,
   0, -1, F_EMPTY|F_WHITE,

  PATTERN, 8, 26,               /*  ~ # O  */
  -1, -1, F_BLACK|F_EMPTY,      /*  # . $  */
  -2, 0, F_BLACK,               /*  ~ ~ ~  */
   1, -1, F_WHITE,              /*    ~ O    */
   1, 0, F_BLACK,
   2, 0, F_WHITE,
   1, 1, F_EMPTY,
   0, 1, F_EMPTY|F_WHITE,
   0, -1, F_EMPTY|F_WHITE,

  PATTERN, 8, 26,               /*  ~ # O  */
  -1, -1, F_BLACK|F_EMPTY,      /*  # . $  */
  -2, 0, F_BLACK,               /*  ~ ~ ~  */
  -1, 0, F_EMPTY,
  -1, 1, F_BLACK,
   0, 1, F_WHITE,
   0, -1, F_BLACK|F_EMPTY,
  -2, -1, F_BLACK|F_EMPTY,
  -2, 1, F_BLACK|F_EMPTY,

  PATTERN, 4, 26,       /*   .   */
  0, 2,  F_EMPTY,       /* #   # */
  -1, 1, F_BLACK,       /*   $ O */
  1, 1,  F_BLACK,
  1, 0,  F_WHITE,

  PATTERN, 4, 24,       /*   .   */
  0, 1, F_EMPTY,        /*   $   */
  -1, -1, F_WHITE,      /* O # O */
  0, -1, F_BLACK,
  1, -1, F_WHITE,

  PATTERN, 4, 24,       /*   .   */
  0, 1, F_EMPTY,        /*   $ O */
  -1, -1, F_WHITE,      /* O #   */
  0, -1, F_BLACK,
  1, 0, F_WHITE,

  PATTERN, 4, 26,       /*   O   */
  0, 1, F_EMPTY,        /* O . O */
  -1, 1, F_WHITE,       /*   $   */
  1, 1, F_WHITE,
  0, 2, F_WHITE,
  PATTERN, 8, 24,         /*     O   */
  -2, 0, F_EMPTY|F_BLACK, /* ~ $ # O */
  0, -1, F_EMPTY,         /*   . . . */
  1, -2, F_OFF,           /*     ~   */
  1, -1, F_EMPTY,
  1, 0, F_BLACK,
  1, 1, F_WHITE,
  2, -1, F_EMPTY,
  1, 0, F_WHITE,

  PATTERN, 8, 28,       /* . # O   */
  -1, 0, F_EMPTY,       /* . $ # O */
  0, -1, F_EMPTY,       /*   . .   */
  0, 1, F_BLACK,
  1, 0, F_BLACK,
  1, 1, F_WHITE,
  2, 0, F_WHITE,
  -1, 1, F_EMPTY,
  1, -1, F_EMPTY,

  PATTERN, 8, 23,       /*   # #   */
  0, 2, F_BLACK,        /*   .   # */
  1, 2, F_BLACK,        /* ~ $ . # */
  2, 0, F_BLACK,        /*   ~     */
  2, 1, F_BLACK,
  1, 0, F_EMPTY,
  0, 1, F_EMPTY,
  -1, 0, F_EMPTY|F_BLACK|F_WHITE,
  0, -1, F_EMPTY|F_WHITE,

  PATTERN, 8, 24,       /*   O O   */
  0, 2, F_WHITE,        /*   .   O */
  1, 2, F_WHITE,        /* ~ $ . O */
  2, 0, F_WHITE,        /*   ~     */
  2, 1, F_WHITE,
  1, 0, F_EMPTY,
  0, 1, F_EMPTY,
  -1, 0, F_EMPTY|F_WHITE|F_BLACK,
  0, -1, F_EMPTY|F_BLACK,

  PATTERN, 8, 34,       /* ~ # # */
  -1, 1, F_BLACK,       /* # . # */
  -1, 0, F_BLACK,       /* # $   */
  0, 2, F_BLACK,        /*   ~   */
  0, 1, F_EMPTY,
  1, 1, F_BLACK,
  1, 2, F_BLACK,
  -1, 2, F_BLACK|F_EMPTY,
  0, -1, F_WHITE|F_EMPTY,

  PATTERN, 7, 34,               /* ~ # ~ */
  -1, 1, F_BLACK,               /* # . # */
  -1, 0, F_BLACK|F_EMPTY,       /* ~ $   */
  -1, 2, F_BLACK|F_EMPTY,
  0, 2, F_BLACK,
  0, 1, F_EMPTY,
  1, 1, F_BLACK,
  1, 2, F_BLACK|F_EMPTY,

  PATTERN, 10, 27,      /*   . . .   */ /*(This is Wally's fuseki for */
  -2, 0, F_BLACK,       /* # . $ . # */ /* a 9 x 9 board with 4 handicap
*/
  2, 0, F_BLACK,        /*   . . .   */ /* stones.)*/
  -1, 0, F_EMPTY,
  1, 0, F_EMPTY,
  -1, 1, F_EMPTY,
  0, 1, F_EMPTY,
  1, 1, F_EMPTY,
  -1, -1, F_EMPTY,
  0, -1, F_EMPTY,
  1, -1, F_EMPTY,

  PATTERN, 6, 25,       /*     # . #   */
  -1, 1, F_BLACK,       /*   ~ . $     */
  -1, -1, F_BLACK,      /*     #       */
  1, 1, F_BLACK,
  0, 1, F_EMPTY,
  -1, 0, F_EMPTY,
  -2, 0, F_EMPTY|F_BLACK|F_OFF,
  PATTERN, 5, 35,       /*     O #   */
  -1, 0, F_EMPTY,       /*   . $ ~   */
  0, 1, F_WHITE,        /*     ~     */
  1, 1, F_BLACK,
  0, -1, F_OFF,
  0, 1, F_BLACK|F_EMPTY,

  PATTERN,  5, 40,      /*   O #     */
  -1, 0, F_EMPTY,       /*   . $ ~   */
  -1, 1, F_WHITE,       /*     ~     */
  0, 1, F_BLACK,
  0, -1, F_OFF,
  1, 0, F_WHITE|F_EMPTY,

  PATTERN, 6, 38,       /*   O . #   */
  -1, 1, F_WHITE,       /*     $ .   */
  1, 1, F_BLACK,        /*     .     */
  0, 1, F_EMPTY,        /*     ~     */
  1, 0, F_EMPTY,
  0, -1, F_EMPTY,
  0, -2, F_OFF|F_BLACK,

  PATTERN, 6, 38,       /*     . #   */
  -1, 0, F_WHITE,       /*   O $ .   */
  1, 1, F_BLACK,        /*     .     */
  0, 1, F_EMPTY,        /*     ~     */
  1, 0, F_EMPTY,
  0, -1, F_EMPTY,
  0, -2, F_OFF|F_BLACK,

/*
some patterns for a 19 x 19 board, which is a little different from
the high handicap 9 x 9 game this program originally played
*/

  PATTERN, 11, 29,        /*     O   */
  -2, 0, F_EMPTY|F_BLACK, /* ~ $ # O */
  0, -1, F_EMPTY,         /*   . . . */
  1, -2, F_OFF,           /*   ~ ~ ~ */
  1, -1, F_EMPTY,         /*     ~   */
  1, 0, F_BLACK,
  1, 1, F_WHITE,
  2, -1, F_EMPTY,
  -1, -2, F_EMPTY|F_OFF,
  0, -2, F_EMPTY|F_OFF,
  1, -2, F_EMPTY|F_OFF,
  1, 0, F_WHITE,

  PATTERN, 12, 38,      /*     ~ #   */
  -1, 0, F_WHITE,       /*   O $ ~ ~ */
  1, 1, F_BLACK,        /*     ~ ~ ~ */
  0, 1, F_EMPTY|F_BLACK,/*     ~ ~ ~ */
  1, 0, F_EMPTY|F_WHITE,/*     ~     */
  0, -3, F_OFF,
  0, -1, F_EMPTY|F_BLACK,
  0, 2,  F_EMPTY|F_BLACK|F_OFF,
  1, -1, F_EMPTY|F_BLACK|F_OFF,
  2, -1, F_EMPTY|F_BLACK|F_OFF,
  0, -2, F_EMPTY|F_BLACK|F_OFF,
  1, -2, F_EMPTY|F_BLACK|F_OFF,
  2, -2, F_EMPTY|F_BLACK|F_OFF,

  PATTERN, 12, 38,      /*     # ~   */
  -1, 0, F_WHITE,       /*   O $ ~ ~ */
  0, 1, F_BLACK,        /*     ~ ~ ~ */
  1, 1, F_EMPTY|F_BLACK,/*     ~ ~ ~ */
  1, 0, F_EMPTY|F_WHITE,/*     ~     */
  0, -3, F_OFF,
  0, -1, F_EMPTY|F_BLACK,
  0, 2,  F_EMPTY|F_BLACK|F_OFF,
  1, -1, F_EMPTY|F_BLACK|F_OFF,
  2, -1, F_EMPTY|F_BLACK|F_OFF,
  0, -2, F_EMPTY|F_BLACK|F_OFF,
  1, -2, F_EMPTY|F_BLACK|F_OFF,
  2, -2, F_EMPTY|F_BLACK|F_OFF,

/*
Let's not just pass when confronted by
an empty 19 x 19 board.. hence the following patterns
*/

  PATTERN, 15, 25,      /*           ~     */
  0, 1, F_EMPTY,        /*           ~ ~   */
  0, 2, F_EMPTY,        /*     . . .       */
  1, 0, F_EMPTY,        /*   . . . .       */
  1, 1, F_EMPTY,        /*   . $ . .       */
  1, 2, F_EMPTY,        /*     . .         */
  2, 0, F_EMPTY,
  2, 1, F_EMPTY,
  2, 2, F_EMPTY,
  4, 3, F_OFF,
  3, 4, F_OFF,
  -1, 0, F_EMPTY,
  -1, 1, F_EMPTY,
  0, -1, F_EMPTY,
  1, -1, F_EMPTY,
  3, 3, F_EMPTY|F_WHITE|F_BLACK,

  PATTERN, 10, 29,
  -1, 0, F_EMPTY,       /*         ~     */
  -1, 1, F_EMPTY,       /*         ~ ~   */
  0, -1, F_EMPTY,       /*   . . .       */
  1, -1, F_EMPTY,       /*   . $ .       */
  0, 1, F_EMPTY,        /*     . .       */
  1, 0, F_EMPTY,
  1, 1, F_EMPTY,
  2, 2, F_EMPTY|F_WHITE|F_BLACK,
  3, 2, F_OFF,
  2, 3, F_OFF,

  PATTERN, 17, 29,      /*     . . .          */
  0, 1, F_EMPTY,        /*     . . .          */
  0, -1, F_EMPTY,       /*   ~ . $ .   ~ ~    */
  1, 0, F_EMPTY,        /*     . . .          */
  -1, 0, F_EMPTY,       /*     . . .          */
  1, 1, F_EMPTY,
  1, -1, F_EMPTY,
  -1, 1, F_EMPTY,
  -1, -1, F_EMPTY,
  -1, 2, F_EMPTY,
  0, 2, F_EMPTY,
  1, 2, F_EMPTY,
  -1, -2, F_EMPTY,
  0, -2, F_EMPTY,
  1, -2, F_EMPTY,
  3, 0, F_WHITE|F_BLACK|F_EMPTY,
  4, 0, F_OFF|F_BLACK,
  -2, 0, F_WHITE|F_EMPTY,


  PATTERN, 12, 31,      /*     . . .     */
  0, 1, F_EMPTY,        /*     . $ .     */
  0, -1, F_EMPTY,       /*   ~ . . . ~   */
  1, 0, F_EMPTY,        /*       ~       */
  -1, 0, F_EMPTY,       /*       ~       */
  1, 1, F_EMPTY,
  1, -1, F_EMPTY,
  -1, 1, F_EMPTY,
  -1, -1, F_EMPTY,
  -2, -1, F_EMPTY|F_BLACK,
  2, -1, F_EMPTY|F_BLACK,
  0, -2, F_EMPTY|F_BLACK|F_WHITE,
  0, -3, F_OFF,

  PATTERN, 22, 33,      /*   ~ ~ #       */
  0, 1, F_EMPTY,        /*   . . . . .   */
  0, -1, F_EMPTY,       /*   . . $ . .   */
  1, 0, F_EMPTY,        /*   . . . . .   */
  -1, 0, F_EMPTY,       /*   . . . . .   */
  1, 1, F_EMPTY,
  1, -1, F_EMPTY,
  -1, 1, F_EMPTY,
  -1, -1, F_EMPTY,
  2, -2, F_EMPTY,
  -2, -2, F_EMPTY,
  0, 2, F_BLACK,
  0, -2, F_EMPTY,
  2, 0, F_EMPTY,
  -2, 0, F_EMPTY,
  2, 1, F_EMPTY,
  2, -1, F_EMPTY,
  -2, 1, F_EMPTY,
  -2, -1, F_EMPTY,
  1, -2, F_EMPTY,
  -1, -2, F_EMPTY,
  -1, 2, F_EMPTY|F_WHITE,
  -2, 2, F_EMPTY|F_WHITE,

  PATTEND
};

